<?php declare(strict_types=1);

namespace BitBadger\PDODocument;

use PDO;

/**
 * Functions to delete documents
 */
class Delete
{
    /**
     * Delete a document by its ID
     *
     * @param string $tableName The table from which the document should be deleted
     * @param mixed $docId The ID of the document to be deleted
     * @param PDO|null $pdo The database connection to use (optional; will obtain one if not provided)
     * @throws DocumentException If any is encountered
     */
    public static function byId(string $tableName, mixed $docId, ?PDO $pdo = null): void
    {
        Custom::nonQuery(Query\Delete::byId($tableName), Parameters::id($docId), $pdo);
    }

    /**
     * Delete documents by matching a comparison on JSON fields
     *
     * @param string $tableName The table from which documents should be deleted
     * @param array|Field[] $fields The field comparison to match
     * @param PDO|null $pdo The database connection to use (optional; will obtain one if not provided)
     * @param string $conjunction How to handle multiple conditions (optional; defaults to `AND`)
     * @throws DocumentException If any is encountered
     */
    public static function byFields(string $tableName, array $fields, ?PDO $pdo = null,
                                    string $conjunction = 'AND'): void
    {
        $namedFields = Parameters::nameFields($fields);
        Custom::nonQuery(Query\Delete::byFields($tableName, $namedFields, $conjunction),
            Parameters::addFields($namedFields, []), $pdo);
    }
}
